package org.ow2.weblab.core.annotator;

/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

import java.net.URI;
import java.net.URISyntaxException;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.factory.AnnotationFactory;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.rdf.RDFReader;
import org.ow2.weblab.rdf.RDFWriter;
import org.ow2.weblab.rdf.Value;
import org.ow2.weblab.rdf.XMLParser;


public abstract class AbstractAnnotator {
	
	/**
	 * The element properties used to write statements in.
	 */
	protected Element element = Element.getDefaultElement();
	
	public static enum Operator {
		ADD, REMOVE, SET, READ, WRITE
	}

	/**
	 * The Pok on which we will write and read statements
	 */
	protected List<PieceOfKnowledge> poks;

	/**
	 * The WebLab Resource that will be annotated.
	 * It can be null since we can annotate subject which are just URI
	 * 
	 * @see #AbstractAnnotator(URI, PieceOfKnowledge)
	 */
	protected Resource resource;

	/**
	 * The subject URI that will be used when writing/reading a statement.
	 */
	protected URI subjectURI;

	/**
	 * State of the inner Annotator
	 */
	protected boolean isInnerAnnotatorStarted = false;

	/**
	 * The original URI from the constructor (either subject or Resource's uri)
	 */
	protected URI originalURI;

	/**
	 * The language currently supported in annotation
	 * If null, all languages are retrieve
	 */
	private String lang = null;

	/**
	 * Set the writing mode of the annotator (ADD/SET)
	 * TODO: add DELET ?
	 */
	private boolean append = true;

	/**
	 * Creates an annotator on the Resource.
	 * It allows to write and read statement (subject, property, object) about this resource on this resource.
	 * 
	 * @param resource
	 *            the resource to wrap.
	 */
	protected AbstractAnnotator(final Resource resource) {
		this.resource = resource;
		try {
			this.subjectURI = new URI(resource.getUri());
		} catch (final URISyntaxException e) {
			LogFactory.getLog(BaseAnnotator.class).error("Illegal resource URI " + resource.getUri(), e);
		}
	}

	/**
	 * Creates an annotator for an URI.
	 * It allows to write and read about the subject in this PoK
	 * 
	 * @param subject
	 *            the subject of interest
	 * @param pok
	 *            the PieceOfKnowledge on which we manage information about the subject
	 */
	protected AbstractAnnotator(final URI subject, final PieceOfKnowledge pok) {
		this.isInnerAnnotatorStarted = true;
		this.resource = null;
		this.subjectURI = subject;
		this.originalURI = subject;
		if (pok != null) {
			this.poks = new LinkedList<PieceOfKnowledge>();
			poks.add(pok);
		}
	}

	/**
	 * Set the element that will contain statements.
	 * By default, the element is an RDF:Description object
	 * @param prefix the prefix of the element
	 * @param uri the uri of the element
	 * @param localName the localName of the element
	 */
	public void setElement(String prefix, String uri, String localName){
		this.element = new Element(prefix, uri, localName);
	}
	
	/**
	 * Set the element that will contain statements.
	 * By default, the element is an RDF:Description object
	 * @param element an Element
	 */
	public void setElement(Element element){
		this.element = element;
	}
	
	/**
	 * Set the default statements container that is an RDF:Description object
	 */
	public void setDefaultElement(){
		this.element = Element.getDefaultElement();
	}
	
	/**
	 * Start a inner annotator on a different subject.
	 * It allows to use an annotator to read/write stament about on other subject.
	 * If an inner annotator is already started, it will be automatically closed
	 * 
	 * @param subject
	 */
	public void startInnerAnnotatorOn(final URI subject) {
		this.isInnerAnnotatorStarted = true;
		this.subjectURI = subject;
	}

	/**
	 * End a inner annotator, the subject will be reseted to the first one
	 * (retrieved when the Annotator has been constructed).
	 */
	public void endInnerAnnotator() {
		if (this.resource != null) {
			this.isInnerAnnotatorStarted = false;
			this.subjectURI = XMLParser.buildURI(this.resource.getUri());
		} else {
			this.subjectURI = this.originalURI;
		}
	}

	/**
	 * Return the writing mode of the annotator:
	 * if the annotator is in append mode, it will add annotations else the annotator will override existing annotations
	 * 
	 * @return true if the annotator will add annotations or false if the annotator will override existing annotations
	 */
	public boolean isAppendMode() {
		return this.append;
	}

	/**
	 * Set the writing mode of the annotator
	 * 
	 * @param append
	 *            true if the annotator will add annotations or false if the annotator will override existing annotations
	 */
	public void setAppendMode(final boolean append) {
		this.append = append;
	}

	/**
	 * Start to support a specific language defined annotation
	 * 
	 * @param language
	 */
	public void startSpecifyLanguage(final String language) {
		this.lang = language;
	}

	/**
	 * End the support for specific language
	 */
	public void endSpecifyLanguage() {
		this.lang = null;
	}


	protected void retrieveAnnotation() {
		if (this.poks != null && !poks.isEmpty() && this.isInnerAnnotatorStarted) {
			return;
		}
		this.poks = new LinkedList<PieceOfKnowledge>();
		if (this.resource.getAnnotation().isEmpty()) {
			poks.add(AnnotationFactory.createAndLinkAnnotation(this.resource));
		} else {
			this.poks.addAll(this.resource.getAnnotation());
		}
	}

	public <T> Value<T> applyOperator(final Operator operator, final String rdfsPrefix, final URI rdfsUri, final String property, final Class<T> clazz, final T val) {
		Value<T> value = null;
		retrieveAnnotation();
		switch (operator) {
			case READ:
				for(PieceOfKnowledge pok:poks){
					Value<T> ivalue = RDFReader.readValueFromStatement(pok, this.subjectURI, rdfsUri, property, this.lang, clazz, element);
					if (ivalue != null){
						if (value == null){
							value = ivalue;
						}else{
							value.aggregate(ivalue);
						}
					}
				}
				break;
			case WRITE:
				try {
					Object object = XMLParser.xmlTransform(val);
					RDFWriter.writeStatement(this.poks.get(0), this.subjectURI, rdfsPrefix, rdfsUri, property, object, this.lang, this.append, element);
				} catch (Exception e) {
					e.printStackTrace();
				}
				break;
			case REMOVE:
				break;
		}
		return value;
	}
	
	
}
