package org.ow2.weblab.rdf;

/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

/**
 * This class simulate several behaviours:
 * <ul>
 * <li>a typed object</li>
 * <li>a list of typed object</li>
 * <li>support a typed object access by language</li>
 * </ul>
 * 
 * @param <T>
 *            the type of the object value.
 */
public class Value<T> implements Iterable<T> {

	protected final LinkedList<T> list = new LinkedList<T>();
	protected final Map<String, T> byLang = new HashMap<String, T>();

	/**
	 * If you have rational to grant a public access to the constructor
	 * please, explain it at forum.weblab-project.org
	 */
	protected Value() {
		//
	}

	/**
	 * If you have rational to grant a public access to the constructor
	 * please, explain it at forum.weblab-project.org
	 */
	protected Value(T item) {
		addItem(item, null);
	}

	/**
	 * If you have rational to give a grant access to this method
	 * please, explain it at forum.weblab-project.org
	 */
	protected void addItem(T item, String lang) {
		this.list.add(item);
		if (lang != null) {
			// TODO: support list of items
			this.byLang.put(lang, item);
		}
	}

	/**
	 * Returns a list of typed values.
	 * One can note that the association of typed values with language attributes will be lost for the returned list.
	 * 
	 * @return a list of typed values
	 */
	@SuppressWarnings("unchecked")
	public LinkedList<T> getValues() {
		return (LinkedList<T>) this.list.clone();
	}

	/**
	 * Returns true if this object contains at least one typed value
	 * 
	 * @return true if this object contains at least one typed value else false
	 */
	public boolean hasValue() {
		return !this.list.isEmpty();
	}

	/**
	 * Returns the first typed value.
	 * 
	 * @return the first typed value or null if there is no value
	 */
	public T firstTypedValue() {
		return firstTypedValue(null);
	}

	/**
	 * Returns the first typed value that is defined this the language attribute.
	 * 
	 * @return the first typed value that is defined this the language attribute or null if there is no value.
	 */
	public T firstTypedValue(String lang) {
		if (this.list.isEmpty()) {
			return null;
		}
		if (lang == null) {
			return this.list.get(0);
		}

		return this.byLang.get(lang);
	}

	/**
	 * Returns the number of typed values available
	 * 
	 * @return the number of typed values available
	 */
	public int size() {
		return this.list.size();
	}

	/**
	 * Returns the toString() method result on the first typed value or null if there is not value available.
	 * 
	 * @return the toString() method result on the first typed value or null if there is not value available.
	 */
	@Override
	public String toString() {
		T value = firstTypedValue();
		if (value == null) {
			return null;
		}
		return value.toString();
	}

	@Override
	public Iterator<T> iterator() {
		return this.list.iterator();
	}
	
	/**
	 * Aggregate two value
	 * @param otherValue
	 * @return
	 */
	public Value<T> aggregate(Value<T> otherValue){
		byLang.putAll(otherValue.byLang);
		list.addAll(otherValue.list);
		return this;
	}
}
